package database.generic;

import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DateFormat;

/**
 * All tests related to date functionality. This class is shared between all
 * databases and for each database we need to derive new class and specify
 * the connection information. 
 */
public class DateTest extends DatabaseTest
{
    /**
     * Create new date test.
     *
     * @param strTestName - name of the test
     * @param strDataSourceName - data source to use
     * @throws NumberFormatException
     * @throws Exception
     */
    public DateTest(
            String strTestName,
            String strDataSourceName
            ) throws NumberFormatException,
            Exception
    {
        super(strTestName, strDataSourceName);
        setUp();
    }

    /**
     * Test insertion of the date to the database.
     * Use the already setup connection and transaction.
     * No need to close the connection since base class is doing it for us.
     *
     * @throws Exception
     */
    public void testInsertAndSelectDate(
            ) throws Exception
    {
        String INSERT_DATE = "insert into date_test(date_test) values (?)";
        String SELECT_DATES = "select * from date_test";

        Date              dtDate = new Date((new java.util.Date()).getTime());
        PreparedStatement insertStatement;
        int               iUpdateCount;

        System.out.println("Inserting date value: "
                + DateFormat.getDateInstance().format(dtDate));
        m_transaction.begin();

        try
        {
            insertStatement = m_connection.prepareStatement(INSERT_DATE);
            insertStatement.setDate(1, (java.sql.Date) dtDate);

            iUpdateCount = executeUpdate(insertStatement, INSERT_DATE);

            System.out.println("Commiting...");
            m_transaction.commit();
        }
        catch(Exception eExc)
        {
            System.out.println("ROLLBACK");
            m_transaction.rollback();
            throw eExc;
        }
        /*
        Assert.assertEquals("Exactly one record shoud have been inserted.",
        iUpdateCount, 1);
        */
        // Now select it back to be sure it is there
        Statement selectStatement = null;
        ResultSet results = null;

        try
        {
            Date dtRetrievedDate;

            selectStatement = m_connection.createStatement();
            results = selectStatement.executeQuery(SELECT_DATES);
            /*
            Assert.assertTrue("The inserted date is not in the database.",
            results.next());
            */
            if (results.next())
                dtRetrievedDate = results.getDate(1);
            else
                dtRetrievedDate = null;
            /*
            Assert.assertNotNull("The inserted date is null in the database", dtRetrievedDate);
            */
// TODO: Fix this
//         Assert.assertEquals("The date retrieved from database "
//                             + DateFormat.getDateTimeInstance().format(dtRetrievedDate)
//                             + " is not the same as the inserted one "
//                             + DateFormat.getDateTimeInstance().format(dtDate),
//                             dtRetrievedDate, dtDate);
            System.out.println("Retrieved date value: "
                    + DateFormat.getDateInstance().format(dtRetrievedDate));



            if (results != null)
            {
                results.close();
                System.out.println("results.close();");
            }

        if (selectStatement != null)
        {
            System.out.println("1 statement.close();");
            selectStatement.close();
            System.out.println("2 statement.close();");
        }
        tearDown();
        } catch(Exception e) {}
    }
}



